@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -29,7 +29,6 @@
 
 #include "gmime-common.h"
 #include "gmime-content-type.h"
-#include "gmime-table-private.h"
 #include "gmime-parse-utils.h"
 
 
@@ -151,7 +150,7 @@ g_mime_content_type_new (const char *type, const char *subtype)
 {
 	GMimeContentType *mime_type;
 	
-	mime_type = g_object_new (GMIME_TYPE_CONTENT_TYPE, NULL);
+	mime_type = g_object_newv (GMIME_TYPE_CONTENT_TYPE, 0, NULL);
 	
 	if (type && *type && subtype && *subtype) {
 		mime_type->type = g_strdup (type);
@@ -194,45 +193,20 @@ GMimeContentType *
 g_mime_content_type_new_from_string (const char *str)
 {
 	GMimeContentType *mime_type;
-	char *type = NULL, *subtype;
 	const char *inptr = str;
+	char *type, *subtype;
 	
 	g_return_val_if_fail (str != NULL, NULL);
 	
-	/* get the type */
-	type = (char *) inptr;
-	while (*inptr && is_ttoken (*inptr))
-		inptr++;
+	if (!g_mime_parse_content_type (&inptr, &type, &subtype))
+		return g_mime_content_type_new ("application", "octet-stream");
 	
-	type = g_strndup (type, (unsigned) (inptr - type));
-	
-	decode_lwsp (&inptr);
-	
-	/* get the subtype */
-	if (*inptr == '/') {
-		inptr++;
-		
-		decode_lwsp (&inptr);
-		
-		subtype = (char *) inptr;
-		while (*inptr && is_ttoken (*inptr))
-			inptr++;
-		
-		if (inptr > subtype)
-			subtype = g_strndup (subtype, (size_t) (inptr - subtype));
-		else
-			subtype = NULL;
-		
-		decode_lwsp (&inptr);
-	} else {
-		subtype = NULL;
-	}
-	
-	mime_type = g_mime_content_type_new (type, subtype);
-	g_free (subtype);
-	g_free (type);
+	mime_type = g_object_newv (GMIME_TYPE_CONTENT_TYPE, 0, NULL);
+	mime_type->subtype = subtype;
+	mime_type->type = type;
 	
 	/* skip past any remaining junk that shouldn't be here... */
+	decode_lwsp (&inptr);
 	while (*inptr && *inptr != ';')
 		inptr++;
 	
