@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -151,8 +151,15 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 	register char *mapptr;
 	ssize_t nread;
 	
-	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
+	if (mstream->fd == -1) {
+		errno = EBADF;
 		return -1;
+	}
+	
+	if (stream->bound_end != -1 && stream->position >= stream->bound_end) {
+		errno = EINVAL;
+		return -1;
+	}
 	
 	/* make sure we are at the right position */
 	mapptr = mstream->map + stream->position;
@@ -178,8 +185,15 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 	register char *mapptr;
 	ssize_t nwritten;
 	
-	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
+	if (mstream->fd == -1) {
+		errno = EBADF;
 		return -1;
+	}
+	
+	if (stream->bound_end != -1 && stream->position >= stream->bound_end) {
+		errno = EINVAL;
+		return -1;
+	}
 	
 	/* make sure we are at the right position */
 	mapptr = mstream->map + stream->position;
@@ -202,7 +216,10 @@ stream_flush (GMimeStream *stream)
 {
 	GMimeStreamMmap *mstream = (GMimeStreamMmap *) stream;
 	
-	g_return_val_if_fail (mstream->fd != -1, -1);
+	if (mstream->fd == -1) {
+		errno = EBADF;
+		return -1;
+	}
 	
 #ifdef HAVE_MSYNC
 	return msync (mstream->map, mstream->maplen, MS_SYNC /* | MS_INVALIDATE */);
@@ -237,7 +254,8 @@ stream_eos (GMimeStream *stream)
 {
 	GMimeStreamMmap *mstream = (GMimeStreamMmap *) stream;
 	
-	g_return_val_if_fail (mstream->fd != -1, TRUE);
+	if (mstream->fd == -1)
+		return TRUE;
 	
 	return mstream->eos;
 }
@@ -247,8 +265,10 @@ stream_reset (GMimeStream *stream)
 {
 	GMimeStreamMmap *mstream = (GMimeStreamMmap *) stream;
 	
-	if (mstream->fd == -1)
+	if (mstream->fd == -1) {
+		errno = EBADF;
 		return -1;
+	}
 	
 	mstream->eos = FALSE;
 	
@@ -261,7 +281,10 @@ stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 	GMimeStreamMmap *mstream = (GMimeStreamMmap *) stream;
 	gint64 real = stream->position;
 	
-	g_return_val_if_fail (mstream->fd != -1, -1);
+	if (mstream->fd == -1) {
+		errno = EBADF;
+		return -1;
+	}
 	
 	switch (whence) {
 	case GMIME_STREAM_SEEK_SET:
@@ -286,11 +309,15 @@ stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 	}
 	
 	/* sanity check the resultant offset */
-	if (real < stream->bound_start)
+	if (real < stream->bound_start) {
+		errno = EINVAL;
 		return -1;
+	}
 	
-	if (stream->bound_end != -1 && real > stream->bound_end)
+	if (stream->bound_end != -1 && real > stream->bound_end) {
+		errno = EINVAL;
 		return -1;
+	}
 	
 	/* reset eos if appropriate */
 	if ((stream->bound_end != -1 && real < stream->bound_end) ||
@@ -305,6 +332,13 @@ stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 static gint64
 stream_tell (GMimeStream *stream)
 {
+	GMimeStreamMmap *mstream = (GMimeStreamMmap *) stream;
+	
+	if (mstream->fd == -1) {
+		errno = EBADF;
+		return -1;
+	}
+	
 	return stream->position;
 }
 
@@ -313,6 +347,11 @@ stream_length (GMimeStream *stream)
 {
 	GMimeStreamMmap *mstream = (GMimeStreamMmap *) stream;
 	
+	if (mstream->fd == -1) {
+		errno = EBADF;
+		return -1;
+	}
+	
 	if (stream->bound_start != -1 && stream->bound_end != -1)
 		return stream->bound_end - stream->bound_start;
 	
@@ -325,16 +364,15 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 	/* FIXME: maybe we should return a GMimeStreamFs? */
 	GMimeStreamMmap *mstream;
 	
-	mstream = g_object_new (GMIME_TYPE_STREAM_MMAP, NULL);
-	mstream->owner = FALSE;
-	mstream->fd = GMIME_STREAM_MMAP (stream)->fd;
-	
-	mstream->map = GMIME_STREAM_MMAP (stream)->map;
-	mstream->maplen = GMIME_STREAM_MMAP (stream)->maplen;
-	
+	mstream = g_object_newv (GMIME_TYPE_STREAM_MMAP, 0, NULL);
 	g_mime_stream_construct (GMIME_STREAM (mstream), start, end);
+	mstream->fd = GMIME_STREAM_MMAP (stream)->fd;
+	mstream->owner = FALSE;
 	
-	return GMIME_STREAM (mstream);
+	mstream->maplen = GMIME_STREAM_MMAP (stream)->maplen;
+	mstream->map = GMIME_STREAM_MMAP (stream)->map;
+	
+	return (GMimeStream *) mstream;
 }
 
 
@@ -346,7 +384,7 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
  *
  * Creates a new #GMimeStreamMmap object around @fd.
  *
- * Returns a stream using @fd.
+ * Returns: a stream using @fd.
  **/
 GMimeStream *
 g_mime_stream_mmap_new (int fd, int prot, int flags)
@@ -357,8 +395,7 @@ g_mime_stream_mmap_new (int fd, int prot, int flags)
 	gint64 start;
 	char *map;
 	
-	start = lseek (fd, 0, SEEK_CUR);
-	if (start == -1)
+	if ((start = lseek (fd, 0, SEEK_CUR)) == -1)
 		return NULL;
 	
 	if (fstat (fd, &st) == -1)
@@ -368,16 +405,15 @@ g_mime_stream_mmap_new (int fd, int prot, int flags)
 	if (map == MAP_FAILED)
 		return NULL;
 	
-	mstream = g_object_new (GMIME_TYPE_STREAM_MMAP, NULL);
+	mstream = g_object_newv (GMIME_TYPE_STREAM_MMAP, 0, NULL);
+	g_mime_stream_construct ((GMimeStream *) mstream, start, -1);
 	mstream->owner = TRUE;
 	mstream->eos = FALSE;
 	mstream->fd = fd;
 	mstream->map = map;
 	mstream->maplen = st.st_size;
 	
-	g_mime_stream_construct (GMIME_STREAM (mstream), start, -1);
-	
-	return GMIME_STREAM (mstream);
+	return (GMimeStream *) mstream;
 #else
 	return NULL;
 #endif /* HAVE_MMAP */
@@ -395,7 +431,7 @@ g_mime_stream_mmap_new (int fd, int prot, int flags)
  * Creates a new #GMimeStreamMmap object around @fd with bounds @start
  * and @end.
  *
- * Returns a stream using @fd with bounds @start and @end.
+ * Returns: a stream using @fd with bounds @start and @end.
  **/
 GMimeStream *
 g_mime_stream_mmap_new_with_bounds (int fd, int prot, int flags, gint64 start, gint64 end)
@@ -417,16 +453,15 @@ g_mime_stream_mmap_new_with_bounds (int fd, int prot, int flags, gint64 start, g
 	if ((map = mmap (NULL, len, prot, flags, fd, 0)) == MAP_FAILED)
 		return NULL;
 	
-	mstream = g_object_new (GMIME_TYPE_STREAM_MMAP, NULL);
+	mstream = g_object_newv (GMIME_TYPE_STREAM_MMAP, 0, NULL);
+	g_mime_stream_construct ((GMimeStream *) mstream, start, end);
 	mstream->owner = TRUE;
 	mstream->eos = FALSE;
 	mstream->fd = fd;
 	mstream->map = map;
 	mstream->maplen = len;
 	
-	g_mime_stream_construct (GMIME_STREAM (mstream), start, end);
-	
-	return GMIME_STREAM (mstream);
+	return (GMimeStream *) mstream;
 #else
 	return NULL;
 #endif /* HAVE_MMAP */
