@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ int main (int argc, char **argv)
 	g_mime_init (0);
 	
 	stream = g_mime_stream_fs_new (fd);
-	istream = g_mime_stream_filter_new_with_stream (stream);
+	istream = g_mime_stream_filter_new (stream);
 	g_object_unref (stream);
 	best = (GMimeFilterBest *) g_mime_filter_best_new (GMIME_FILTER_BEST_CHARSET |
 							   GMIME_FILTER_BEST_ENCODING);
@@ -56,13 +56,13 @@ int main (int argc, char **argv)
 	printf ("summary for %s:\n", argv[1]);
 	printf ("\tbest charset: %s\n", g_mime_filter_best_charset (best));
 	printf ("\tbest encoding (7bit): %s\n",
-		g_mime_part_encoding_to_string (
+		g_mime_content_encoding_to_string (
 			g_mime_filter_best_encoding (best, GMIME_BEST_ENCODING_7BIT)));
 	printf ("\tbest encoding (8bit): %s\n",
-		g_mime_part_encoding_to_string (
+		g_mime_content_encoding_to_string (
 			g_mime_filter_best_encoding (best, GMIME_BEST_ENCODING_8BIT)));
 	printf ("\tbest encoding (binary): %s\n",
-		g_mime_part_encoding_to_string (
+		g_mime_content_encoding_to_string (
 			g_mime_filter_best_encoding (best, GMIME_BEST_ENCODING_BINARY)));
 	
 	g_object_unref (istream);
