@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,10 +30,8 @@
 
 #include "testsuite.h"
 
-#ifndef G_OS_WIN32
-#define ENABLE_ZENTIMER
+/*#define ENABLE_ZENTIMER*/
 #include "zentimer.h"
-#endif
 
 #ifdef TEST_CACHE
 static char *charsets[] = {
