@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,15 +32,15 @@
 
 #include <gmime/gmime.h>
 
-#ifndef G_OS_WIN32
+#if !defined (G_OS_WIN32) || defined (__MINGW32__)
 #define ENABLE_ZENTIMER
 #include "zentimer.h"
 #endif
 
 /*#define TEST_RAW_HEADER*/
-#define TEST_PRESERVE_HEADERS
-#define PRINT_MIME_STRUCT
-/*#define TEST_WRITE_TO_STREAM*/
+//#define TEST_PRESERVE_HEADERS
+//#define PRINT_MIME_STRUCT
+#define TEST_WRITE_TO_STREAM
 
 static void
 print_depth (int depth)
@@ -90,16 +90,18 @@ print_mime_struct (GMimeObject *part, int depth)
 }
 
 static void
-test_parser (GMimeStream *stream)
+test_parser (GMimeStream *istream)
 {
-	GMimeParser *parser;
+	GMimeStream *fstream, *stream;
 	GMimeMessage *message;
+	GMimeParser *parser;
+	GMimeFilter *filter;
 	char *text;
 	
-	fprintf (stdout, "\nTesting MIME parser...\n\n");
+	//fprintf (stdout, "\nTesting MIME parser...\n\n");
 	
 	parser = g_mime_parser_new ();
-	g_mime_parser_init_with_stream (parser, stream);
+	g_mime_parser_init_with_stream (parser, istream);
 	
 	ZenTimerStart (NULL);
 	message = g_mime_parser_construct_message (parser);
@@ -116,37 +118,34 @@ test_parser (GMimeStream *stream)
 	g_free (text);
 	
 #ifdef TEST_RAW_HEADER
-	{
-		char *raw;
-		
-		raw = g_mime_message_get_headers (message);
-		fprintf (stdout, "\nTesting raw headers...\n\n%s\n", raw);
-		g_free (raw);
-	}
+	text = g_mime_message_get_headers (message);
+	fprintf (stdout, "\nTesting raw headers...\n\n%s\n", text);
+	g_free (text);
 #endif
 	
 #ifdef TEST_PRESERVE_HEADERS
-	{
-		GMimeStream *stream;
-		
-		g_mime_header_list_set_stream (GMIME_OBJECT (message)->headers, NULL);
-		
-		fprintf (stdout, "\nTesting preservation of headers...\n\n");
-		stream = g_mime_stream_file_new (stdout);
-		/*g_mime_header_set_stream (GMIME_OBJECT (message)->headers, NULL);*/
-		g_mime_header_list_write_to_stream (GMIME_OBJECT (message)->headers, stream);
-		g_mime_stream_flush (stream);
-		GMIME_STREAM_FILE (stream)->fp = NULL;
-		g_object_unref (stream);
-		fprintf (stdout, "\n");
-	}
+	g_mime_header_list_set_stream (GMIME_OBJECT (message)->headers, NULL);
+	
+	fprintf (stdout, "\nTesting preservation of headers...\n\n");
+	stream = g_mime_stream_file_new (stdout);
+	/*g_mime_header_set_stream (GMIME_OBJECT (message)->headers, NULL);*/
+	g_mime_header_list_write_to_stream (GMIME_OBJECT (message)->headers, stream);
+	g_mime_stream_flush (stream);
+	GMIME_STREAM_FILE (stream)->fp = NULL;
+	g_object_unref (stream);
+	fprintf (stdout, "\n");
 #endif
 	
 #ifdef TEST_WRITE_TO_STREAM
-	stream = g_mime_stream_fs_new (2);
-	g_mime_object_write_to_stream (GMIME_OBJECT (message), stream);
-	g_mime_stream_flush (stream);
+	stream = g_mime_stream_fs_new (1);
+	fstream = g_mime_stream_filter_new (stream);
+	filter = g_mime_filter_crlf_new (TRUE, TRUE);
+	g_mime_stream_filter_add (GMIME_STREAM_FILTER (fstream), filter);
+	g_object_unref (filter);
+	g_mime_object_write_to_stream (GMIME_OBJECT (message), fstream);
+	g_mime_stream_flush (fstream);
 	GMIME_STREAM_FS (stream)->fd = -1;
+	g_object_unref (fstream);
 	g_object_unref (stream);
 #endif
 	
